/**
 * Filename: client_listener.h
 *
 * Description: Implementation the client_listener module. This module
 * will be responsiable for listening and dispatching all tasks 
 *
 * Original Author: Nathan Boeger
 *
 * $Id: client_listener.cpp,v 1.5 2002/05/07 06:50:51 nathan Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/resource.h>

// my includes 
#include "khneia.h"
#include "khneia_protocol.h"
#include "client_sys_conf.h"
#include "client_register.h"
#include "system_info.h"
#include "host_info.h"
#include "disk_stats.h"
#include "system_stats.h"
#include "client_listener.h"
#include "flog.h"


client_listener::client_listener()
{
	error = K_BAD;
	sys_conf = NULL;
}


/*
 * Construct the listener and make sure we have a valid 
 * client_sys_conf module. Otherwise keep the class error
 * flagged !
 */
client_listener::client_listener(client_sys_conf *sys)
{
	if( sys )
	{
		sys_conf = sys;
		error = K_GOOD;
	}
	else
	{
		K_ERR_LOG("client_listener constructor, junk data sys !");
	}
}

/*
 * run
 * we need to create our socket and start to listen. 
 */
int client_listener::run()
{
	int err = error, n = 1;
	size_t len;
	char *buff = NULL, *base = NULL;
	struct sockaddr_in sock, client;
	struct khneia_header_pkt header;

	if( (err != K_BAD) &&  (sfd = socket(AF_INET, SOCK_STREAM, 0)) > 0 )
	{
		/*
		 * set up our signal handlers
		 */
		signal(SIGCHLD, child_cleanup);

		/* 
		 * check the sys_conf module to make sure we can get the
		 * valid opts
		 */
		memset(&sock, 0, sizeof(struct sockaddr_in) );
		sock.sin_family = AF_INET;
		sock.sin_port = htons(sys_conf->getListenPort());

		/*
		 * Set our opts 
		 */
		if( setsockopt(sfd, SOL_SOCKET, SO_REUSEADDR, &n, sizeof(int)) >= 0 )
		{
			/*
			 * Bind to the socket, but do not be nice.. just loop untill we get it !
			 */
			while( bind(sfd, (struct sockaddr *) &sock, sizeof(struct sockaddr_in)) < 0 )
			{
				K_ERR_LOG("Cannot bind !");
				sleep(3);
			}

			while( 1 )
			{
				if( listen(sfd, MAX_CONNECTIONS) >= 0 )
				{
					len = sizeof(struct sockaddr_in);
					// now accept the socket
					nsfd = accept(sfd, (struct sockaddr *)&client, &len);

					if( client.sin_addr.s_addr == sys_conf->getServer() )
					{
						// read from it, check the size read
						memset(&header, 0, sizeof(struct khneia_header_pkt));
						/* 
						 * now recieve the packet... should check the size read..
						 * but will add this later
						 */
						len = recv(nsfd, &header, sizeof(struct khneia_header_pkt), 0);

						NW_TO_HL(header.length);
						NW_TO_HL(header.sequence);

						/*
						 * handle the situation where our packet is larger than just the
						 * header
						 */
						if( header.length == (sizeof(struct khneia_header_pkt)) )
						{
							// just handle the packet
							err = handle_pkt((char *)&header);
						}
						else if( header.length > (sizeof(struct khneia_header_pkt)) )
						{
							// create our buff to handle the data 
							buff = (char *)malloc(header.length);
							if( buff != NULL )
							{
								base = buff;
								// copy over the header
								memcpy(buff, &header, sizeof(struct khneia_header_pkt) );

								// advance our pointer
								buff += sizeof(struct khneia_header_pkt);
								// get the rest
								len = recv(nsfd, buff, header.length, 0);
								// now handle the packet
								err = handle_pkt((char *)&header);
								/* free our buffer if its valid */
								free(base);
								base = NULL;
							}
							else
							{
								K_ERR_LOG("Error, buf is null, cannot allocate memory !!");
							}

						} // if( header.length)
						else
						{
							K_LOG("Did not recieve a valid packet ! going to throw it away ");
						}

					}// if(server == getServer)
					else
					{
						K_LOG("Error ! Invalid host connected to us !");
					}

				} // if(listen)
				else
				{
					K_LOG("CLIENT RUN ERROR setsockopts failed");
				} 

				/* shutdown the socket.. */
				shutdown(nsfd, SHUT_RDWR);

			}// while (1)

		} // if (socket)
		else
		{
			K_LOG("Socket returned error : ");
		}
	}
	/* never reachec */
	close();

	return(err);
}

/*
 * close 
 *
 * This will close the socket and clean up before
 * returing
 */
int client_listener::close()
{
	shutdown(sfd, SHUT_RDWR);
	return(error);
}

/*
 * handle_pkt
 *
 * This will take the pkt in the child process from listen, fork
 * and then service the packet. Note we do not care about process
 * accounting so we discard the pid for the child
 */

int client_listener::handle_pkt(char *pkt)
{
	int err = error;
	struct khneia_header_pkt *head = (struct khneia_header_pkt *)pkt;
	system_info *info = NULL;

	// now fork and service the packet
	if( (error != K_BAD) && (pkt) && (fork() == 0 ) )
	{
		// set back the signals to the default, incase of a rsync packet etc...
		signal(SIGCHLD, SIG_DFL);
		// check the type and create the proper reply
		switch( head->type )
		{
			case SYSTEM_STATS_PKT:
				info = new system_stats(head, sys_conf);
				break;
			case HOST_INFO_PKT:
				info =  new host_info(head, sys_conf);
				break;
			case DISK_INFO_PKT:
				info = new disk_stats(head, sys_conf);
				break;
			default:
				K_LOG("Error, unknown packet !!");
				info = NULL;
				break;
		}
		/* now run and send the packet */
		if( info )
		{
			info->run();
			info->send();
			err = K_GOOD;
			delete info;
		}
		info = NULL;
		exit(0);
	} // if
	// not reached in child
	return(err);
}


/*
 * This will gather the childrens process info
 */
void client_listener::child_cleanup(int sig)
{
	if( sig == SIGCHLD )
	{
		/* just discard for now.. add more code later if needed */
		waitpid(-1, (int *)NULL, 0);
	}
}

/*
 * destructor
 */
client_listener::~client_listener()
{
	// no dynamic memory
}

