/**
 * Filename: client_register.cpp
 *
 * Description: Implementation file for the client_register module
 *
 * Original Author: Nathan Boeger
 *
 * $Id: client_register.cpp,v 1.3 2002/04/26 08:31:06 nathan Exp $
 *
 */

#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "khneia.h"
#include "khneia_protocol.h"
#include "client_sys_conf.h"
#include "client_register.h"
#include "system_info.h"
#include "host_info.h"
#include "disk_stats.h"
#include "system_stats.h"
#include "flog.h"

/*
 * flag the error, we need at least the parser
 */
client_register::client_register()
{
	error = K_BAD;
}

/*
 * Prefered constructor
 * at this point we only need to set the error good and set the 
 * value of the client_sys_conf module
 */
client_register::client_register(client_sys_conf *s)
{
	if( s )
	{
		sys_conf = s;
		error = K_GOOD;
	}
	else
	{
		error = K_BAD;
	}
}


/*
 * run
 *
 * This will bring the register to life, the fist and only task is to fork
 * and then register with the master deamon. This registration will take place
 * after we get our needed packets ready !
 */
int client_register::run()
{
	int i, err = error;
	int pkts[REGISTER_NUM] = {HOST_INFO_PKT, SYSTEM_STATS_PKT, DISK_INFO_PKT};
	khneia_header_pkt head;
	system_info *info = NULL;

	/* check our error state ! */
	if( err != K_BAD )
	{
		/* now loop through the packets and send them */
		for(i = 0; i < REGISTER_NUM; i++)
		{
			head.length = sizeof(struct khneia_header_pkt);
			head.type = pkts[i];
			head.sequence = REGISTER_SEQ;

			switch( head.type )
			{
				case SYSTEM_STATS_PKT:
					info = new system_stats(&head, sys_conf);
					break;
				case HOST_INFO_PKT:
					info =  new host_info(&head, sys_conf);
					break;
				case DISK_INFO_PKT:
					info = new disk_stats(&head, sys_conf);
					break;
			}
			/*  now run and send the packet */
			if( info )
			{
				info->run();
				info->send();
				err = K_GOOD;
				delete info;
			}
			info = NULL;
			memset(&head, 0, sizeof(struct khneia_header_pkt) );
		} // for()
	} // if (error)
	else
	{
		K_ERR_LOG("register_host, class error is flaged !! cannot run !! ");
	}

	return(err);
}


client_register::~client_register()
{
	// no dynamic memory.. do nothing
}

