/**
 * Filename: client_sys_conf.cpp
 *
 * Description: client_sys_conf implementation, note that this
 * file is *not* directly compiled.. it is manifested
 * into the sys_conf.l file !!!
 *
 * Original Author: Nathan Boeger
 *
 * $Id: client_sys_conf.cpp,v 1.5 2002/04/28 06:06:52 nathan Exp $
 *
 */


/*
 * Default constructor
 *
 * This will just set the serverport and
 * the listen port to the defaults, and flag
 * the error to good
 */
client_sys_conf::client_sys_conf()
{
	error = K_GOOD;
	serverport = SERVERPORT;
	listenport = CLIENTPORT;
	alreadyrun = NOT_RAN;
}

/*
 * run
 * ---
 *
 * This will the lexer and gather the options. If it is
 * sucessfull it returns K_GOOD, otherwise K_BAD
 */
int client_sys_conf::run()
{
	FlexLexer* lexer;
	ifstream *in;
	ofstream *out;

	if( alreadyrun == NOT_RAN )
	{
		in = new ifstream(CONF_PATH);
		out = new ofstream(OUT_FILE, ios::nocreate);

		// this will allow us to have multiple instances of this class
		sys_lexer = this; 

		if( !in->bad() && !out->bad() )
		{
			if( error != K_BAD )
			{
				lexer = new client_sys_confFlexLexer(in, out);
				if( lexer )
				{
					while( lexer->yylex() != 0 ) ;
					delete lexer;
				}
				alreadyrun = RAN;
			}
		}

		/* 
		 * return the class error
		 * 
		 * when the lexer runs it will populate data by calling
		 * the set methods. If they have errors they will flag
		 * the class error
		 */
	}
	return(error);
}

/*
 * getlistenPort
 *
 * This will return the port that we need to listen
 * on, The default is 8889
 */

int client_sys_conf::getListenPort()
{
	return(listenport);
}

/*
 * setListenPort
 *
 * This will set the clients listen port. Default is 8889
 */
int client_sys_conf::setListenPort(unsigned int p)
{
	listenport = p;
	return(listenport);
}

/*
 * unset_ran
 * unset the ran flag so the class can run agian
 */
 void client_sys_conf::unset_ran()
 {
	 if(alreadyrun != NOT_RAN)
	 {
		 alreadyrun = NOT_RAN;
	 }
 }

/*
 * getServerPort
 *
 * This will return the server port that we need to connect to.
 * The default is 8888
 */
int client_sys_conf::getServerPort()
{
	return(serverport);
}

/*
 * setServerPort()
 *
 * This will set the server port that we need to connect to
 * The default is 8888
 */
int client_sys_conf::setServerPort(unsigned int p)
{
	serverport = p;
	return(serverport);
}

/*
 * getServer
 *
 * This will return the IP address of the server that we need
 * to connect to.
 */
in_addr_t client_sys_conf::getServer()
{
	return(server);
}

/*
 * setServer(in_addr_t s)
 * sets the server to the ip address s
 */
int client_sys_conf::setServer(in_addr_t s)
{
	server = s;
	return(error);
}

/*
 * setServer(char *s)
 *
 * This will take the servername s and convert it to
 * an IP address. Note that if the name does not 
 * resolve then we throw the error for the whole
 * class since the client dannot work without a master !
 * Otherwise its just good
 * 
 */
int client_sys_conf::setServer(char *s)
{
	struct hostent *hinfo = NULL;

	if( s )
	{
		hinfo = gethostbyname(s);
		if( hinfo )
		{
			// now set the server from the info
			memcpy(&server, hinfo->h_addr_list[0], hinfo->h_length );
			error = K_GOOD;
		}
		else
		{
			error = K_BAD;
		}
	}

	return(error);
}

/*
 * getRsyncPath
 *
 * This will return the path to the rsync bindary
 */
char *client_sys_conf::getRsyncPath()
{        
	return(rsyncpath);
}

/*
 * setRsyncPath
 *
 * This will set the path to the rsync binary
 */
int client_sys_conf::setRsyncPath(char *p)
{
	/* check the pointers !! */
	if( p )
	{
		memset(rsyncpath, 0, MAX_PATH_LEN);
		strncat(rsyncpath, p, strlen(p) );
	}
	else
	{
		error = K_BAD;
	}

	return(error);
}

/*
 * Destructor
 */
client_sys_conf::~client_sys_conf()
{
	// no memory allocated
}

