%{
/**
 * Filename: sys_conf.l
 *
 * Description: the lexer used to parse the khneia
 * client conf file. This will manifest the files
 * that implement the class !
 *
 * Original Author: Nathan Boeger
 *
 * $Id: client_sys_conf.l,v 1.3 2002/04/08 05:29:03 nathan Exp $
 *      
 */

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <fstream.h>
#include <string.h>
#include <sys/types.h>
#include <syslog.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>


 // my includes
#include "khneia.h"
// the  client_conf header
#include "client_sys_conf.h"

client_sys_conf *sys_lexer;

%}

%option prefix="client_sys_conf"
%option noyywrap

server     [ \t]*[sS][eE][rR][vV][eE][rR][ \t]*"="
s_port     [ \t]*[sS][eE][rR][vV][eE][rR][pP][oO][rR][tT][ \t]*"="
l_port     [ \t]*[lL][iI][sS][tT][eE][nN][pP][oO][rR][tT][ \t]*"="
rsync_p    [ \t]*[rR][sS][yY][nN][cC][pP][aA][tT][hH][ \t]*"="

ip         ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)
val        [0-9a-zA-Z\./\-_]*
hostname   ([0-9a-zA-Z\._\-]*)
port       [0-9]+
comment    \n?|([ \t]*)?#.*\n
endval     .*\n    

%x GET_SERVER
%x GET_S_PORT
%x GET_L_PORT
%x GET_R_PATH
%x GET_PORT
%x GET_IP
%x GET_VAL
%x END_VAL 

%%


^{comment}         { ; /* ignore comments */ }
^{server}          { BEGIN GET_SERVER;   }
<GET_SERVER>{ip}   { sys_lexer->setServer( inet_addr(yytext) ); BEGIN END_VAL; }
<GET_SERVER>{hostname}  {  sys_lexer->setServer( yytext);       BEGIN END_VAL; }
^{s_port}          { BEGIN GET_S_PORT; }
<GET_S_PORT>{port} { sys_lexer->setServerPort( atoi(yytext) );  BEGIN END_VAL; }
^{l_port}          { BEGIN GET_L_PORT; }
<GET_L_PORT>{port} { sys_lexer->setListenPort( atoi(yytext) );  BEGIN END_VAL; }
^{rsync_p}         { BEGIN GET_R_PATH; }
<GET_R_PATH>{val}  { sys_lexer->setRsyncPath( yytext );         BEGIN END_VAL; }
<END_VAL>{endval}  { BEGIN 0; }
.|\n               { ; /* Ignore white space */ }
%%

// the client_conf implementation
#include "client_sys_conf.cpp"

