/**
 * Filename: driver.cpp
 *
 * Description: Driver file for testing the classes
 *
 * Original Author: Nathan Boeger
 *
 * $Id: driver.cpp,v 1.16 2002/05/07 06:50:51 nathan Exp $
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <setjmp.h>

#include "khneia.h"
#include "client_sys_conf.h"
#include "client_listener.h"
#include "client_register.h"
#include "flog.h"

/* 
 * Signal handler for SIGHUP
 */
static void reset(int sig);
void client();

client_sys_conf *sys = NULL;
client_register *reg = NULL;
client_listener *lis = NULL;

/* 
 * save our program enviroment for when we 
 * recieve a signal (in our case here HUP)
 */
jmp_buf jbuf;

/*
 * main
 * just set up the signals and then call the client
 * function
 */
int main()
{

	if( fork() == 0 )
	{
		/* make me a deamon */
		setsid();
		/* work from root to make things simple :-) */
		chdir("/");
		/* we catch SIGHIUP and reset the system */
		signal(SIGHUP, reset);

		/*
		 * Check to see if we have already 
		 * had a signal !
		 */
		if( setjmp(jbuf) == 0 )
		{
			client();
		}
		else
		{
		  /*
			* To reset the system (conf file changes etc... )
			* just delete our current class instances and call
			* listens close to cleanup the socket. This will
			* be executed only after a return from a signal
			*/

			lis->close();

			delete lis;
			delete reg;
			delete sys;

			lis = NULL;
			reg = NULL;
			sys = NULL;

			client();
		}

	}

	return(0);
}

/*
 * client
 * We need to run the register and the listener, on startup
 * or after the SIGHUP
 */
void client()
{
	sys = new client_sys_conf();

	if( (sys) && (sys->run() >= 0) )
	{
		reg = new client_register(sys);
		lis = new client_listener(sys);
		if( (reg) && (lis) )
		{
			reg->run();
			lis->run();
		}
	}
}

/*
 * reset
 * All we need to do is just call longjmp and that
 * will return our process from this signal to the
 * main program where we originall called setjmp
 */
static void reset(int sig)
{
	longjmp(jbuf, 1);
}
