/**
 * Filename: host_info.h
 *
 * Description: Derruved class to gather host information such ash
 *    1) hostname
 *    2) os information
 *    3) khniea version
 *    4) default route
 *    5) network ip/netmask/mac addresses
 *
 * Original Author: Nathan Boeger
 *
 * $Id: host_info.cpp,v 1.10 2002/05/08 06:11:06 nathan Exp $
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/udp.h>
#include <netinet/ip_var.h>
#include <netinet/udp_var.h>
#include <net/if_types.h>
#include <sys/sockio.h>
#include <ifaddrs.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/sysctl.h>
#include <net/if.h>
#include <net/if_dl.h>
#include <net/route.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <sys/uio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <sys/stat.h> 
#include <stdlib.h>


/* silly macro */
#define MAX(x,y) x > y ? x : y

/* my includes */
#include "flog.h"
#include "khneia.h"
#include "khneia_protocol.h"
#include "khneia_sender.h"
#include "client_sys_conf.h"
#include "system_info.h"
#include "host_info.h"

/* does nothing, just flags error */
host_info::host_info()
{
	head = NULL;
	error = K_BAD;
	cnt  = 0;
}

host_info::host_info(khneia_header_pkt *h, client_sys_conf *s)
{
	head = NULL;
	cnt  = 0;
	if( (h) && (s) )
	{
		sys = s;
		info = new host_info_pkt;
		if( info )
		{
			info->header.length = sizeof(struct host_info_pkt);
			info->header.sequence = h->sequence;
			info->header.type = h->type;
			error = K_GOOD;
			// set the header now !
			header = (struct khneia_header_pkt *)info;
		}
	}
	else
	{
		K_LOG("Constructor error, invalid data!");
		error = K_BAD;
	}
}


/*
 * run
 *
 * Just call all the proper methods to create
 * our nice pretty packet !
 */
int host_info::run()
{
	if( error != K_BAD )
	{
		setOsInfo();
		setKhneiaVersion();
		setDefaultRoute();
		// lastly get our network info
		setNetworkInfo();
	}

	return(error);
}


/*
 * setOsInfo
 *
 * This will collect the OS information, the only concern
 * is that we do not exceed the max length for the osinfo
 * field
 */
int host_info::setOsInfo()
{
	int mib[2], err = error; 
	char *kernel, *buff;  
	size_t len;

	if( error != K_BAD )
	{
		/* set the correct mib values */
		mib[0] = CTL_KERN; 
		mib[1] = KERN_VERSION;

		/* now call sysctl with a null to get the total length */
		sysctl(mib, 2, NULL, &len, NULL, 0);

		/* create some space */
		kernel = (char *) alloca(len);
		if( kernel )
		{
			memset(kernel, 0, len);
			// call sysctl to get the proper value
			sysctl(mib, 2, kernel, &len, NULL, 0);
			memset(info->osinfo, 0, MAX_OSINFO_LEN);
			// also get rid of those nasty newlines if any
			while( (buff = strchr(kernel, '\n')) )
				*buff = ' ';

			strncpy(info->osinfo, kernel, MAX_OSINFO_LEN);
		}
		else
		{
			K_LOG("Error cannot allocate memory !");
		}
	}

	return(err);
}

/*
 * setKhnieaVersion
 *
 * this will set the khneia version to the packet
 */

int host_info::setKhneiaVersion()
{

	if( error != K_BAD )
	{
		// null the file
		memset(info->khneia_ver, 0, MAX_VERSION_LEN);
		strncat(info->khneia_ver, KHNEIA_VERSION, MAX_VERSION_LEN);
		// now can we add the date ??
		if( (strlen(__DATE__) + strlen(KHNEIA_VERSION)) < MAX_VERSION_LEN )
		{
			strncat(info->khneia_ver, __DATE__, strlen(__DATE__));
		}
	}

	return(error);
}

/*
 * setDefaultRoute
 *
 * This will get the default route and then write it to the packet
 * note that this uses the sysctl call. So it can be executed by
 * any user.  Also.. thanks Mr, Stevens :-)
 */
int host_info::setDefaultRoute()
{
	int mib[6] , err = K_BAD;
	size_t len; 
	char *buf, *next, *foo;
	struct rt_msghdr *rtm;
	struct sockaddr_in *gw;

	/*
	 * Set up the mib info
	 */
	mib[0] = CTL_NET;
	mib[1] = AF_ROUTE;
	mib[2] = 0;
	mib[3] = AF_INET;
	mib[4] = NET_RT_DUMP;
	mib[5] = 0;

	/* get the lenth */
	if( sysctl(mib, 6, NULL, &len, NULL, 0) < 0 )
	{
		K_ERR_LOG("Cannot sysctl on setDefaultRoute");
	}
	else
	{
		/* allocate memory and call agian */
		buf = (char *) alloca(len);
		if( (buf) && (sysctl(mib, 6, buf, &len, NULL, 0) < 0) )
		{
			K_ERR_LOG("Cannot sysctl on setDefaultRoute");
		}
		else
		{
			/*
			 * Need to loop through the message and extract the routes
			 */
			rtm = (struct rt_msghdr *)buf;
			for( next = buf; next < (buf + len); next += rtm->rtm_msglen )
			{
				rtm = (struct rt_msghdr *)next;
				/* check to see if its a gateway */
				if( rtm->rtm_addrs | RTA_GATEWAY )
				{
					foo = (next + sizeof(struct rt_msghdr));
					gw = (struct sockaddr_in *) foo;
					/* 
					 * The kernel keeps the destination as 0.0.0.0, so test for that
					 * after we check to see if its AF_INET family
					 */
					if( (gw->sin_family == AF_INET) && (gw->sin_addr.s_addr == 0) )
					{
						foo += sizeof( struct sockaddr_in);
						gw = (struct sockaddr_in *) foo;
						strncat(info->def_route, inet_ntoa(gw->sin_addr), MAX_IPADDR);
						//info->def_route = gw->sin_addr.s_addr;
						err = K_GOOD;
					}
				}
			} // for

		}// if(sysctl)

	} // if(sysctl)

	return(err);
}

/*
 * setNetworkInfo
 * This will use the sysctl to get the local interfaces. The reason
 * to use sysctl is that its more protable than ioctl calls. Here 
 * agian thanks Mr. Stevens :-)
 */
int host_info::setNetworkInfo()
{
	int sd = 0, err = K_BAD, n;
	size_t len, last_len;
	char *buf = NULL, *bar, *ptr, *tmp;
	struct ifconf ifc;
	struct ifreq *ifr, ifrcopy;
	struct sockaddr_in *sin;
	struct sockaddr_dl *sdl;
	struct network_info info;

	/* give a good guess as to the needed size */
	len = 128 * sizeof(struct ifreq); 
	/* create a socket to ioclt */
	sd = socket(AF_INET, SOCK_DGRAM, 0);

	/* zero out our info struct */
	memset(&info, 0, sizeof(struct network_info));

	/*
	 * Now loop untill no more devices, each time we will
	 * up the buffer and ad more space 
	 */
	last_len = 0;
	cnt = 0;
	if( sd > 0 )
	{
		for( ; ; )
		{
			// use alloca so we do not need to worry about freeing !
			buf = (char *) alloca(len);
			ifc.ifc_len = len;
			ifc.ifc_buf = buf;

			/* get the table untill no more */
			if( ioctl(sd, SIOCGIFCONF, &ifc ) < 0 )
			{
				if( errno != EINVAL || last_len != 0 )
				{
					K_ERR_LOG("IOCTL faild in setNetworkInfo");
				}
			}
			else
			{
				if( ifc.ifc_len == (int)last_len )
					break;

				last_len = ifc.ifc_len;
			}

			len +=  sizeof(struct ifreq);
		}               
	}

	/*
	 * now that we have the interfaces.. we need to go through them and
	 * find our network info 
	 */
	ptr = buf;
	while( ptr < (buf + ifc.ifc_len) )
	{
		ifr = (struct ifreq *)ptr;
		sdl = (struct sockaddr_dl *)&ifr->ifr_addr;

		/*
		 * The first structure we get is the datalink, so get the mac
		 * address and then store it for IP & Netmask address, Note that
		 * every address will have to have a mac address and we will 
		 * report all IP's (so the mac will be dupilcated on aliased interfaces)
		 */
		memcpy(&ifrcopy, ifr, sizeof(struct ifreq) ); 
		if( sdl->sdl_type == IFT_ETHER )
		{
			if( ((n = sdl->sdl_alen) > 0) && (sdl->sdl_alen < MAX_MAC_LEN) )
			{
				/* cute way to set the address where the mac starts */
				tmp = (char *)LLADDR(sdl);
				bar = info.mac;
				while( --n >= 0 )
				{
					sprintf(bar ,"%02x%c", (*tmp++ & 0xff), n > 0 ? ':' : ' '); 
					bar += 3;
				}
			}
		}

		/*
		 * Now call ioctl agian and this time get the device flags, we need
		 * to test for the loopback interface (we do not use it)
		 */
		if( ifr->ifr_addr.sa_family == AF_INET )
		{
			if( (ioctl(sd, SIOCGIFFLAGS, &ifrcopy) >= 0 ) &&
				 !(ifrcopy.ifr_flags & IFF_LOOPBACK) )
			{
				sin = (struct sockaddr_in *) &ifr->ifr_addr;
				memset(info.ipaddr, 0, MAX_IPADDR);
				strncat(info.ipaddr, inet_ntoa(sin->sin_addr), MAX_IPADDR);

				strncpy(info.dev, ifr->ifr_name, MAX_DEV_LEN);
				/* get the netmask convert it and then copy into netmsk */
				memcpy(&ifrcopy, ifr, sizeof(struct ifreq) ); 

				/* call ioctl agian to get the netmask for this interface */
				if( ioctl(sd, SIOCGIFNETMASK, &ifrcopy) >= 0 )
				{
					sin = (struct sockaddr_in *) &ifrcopy.ifr_addr;
					memset(info.netmask, 0, MAX_IPADDR);
					strncat(info.netmask, inet_ntoa(sin->sin_addr), MAX_IPADDR);
					addNetEntry(&info);
				}
			}
		}

		/*
		 * Since newer systems have sa_len. we must account for that
		 */
		len = MAX( sizeof( struct sockaddr), ifr->ifr_addr.sa_len);
		ptr += sizeof(ifr->ifr_name) + len;

	} // while (ptr < buf)

	/* close the socket, we are done now */
	shutdown(sd , SHUT_RDWR);
	/* now assemble the data */
	makePacket();

	return(err);
}

/*
 * addNetEntry
 * 
 * Just create the link list of network_info structs and update
 * the cnt
 */
void host_info::addNetEntry(struct network_info *n)
{
	struct nlist *foo = new nlist;

	if( n )
	{
		// copy the data over to foo
		memcpy(&foo->info, n, sizeof(struct network_info) );
		if( head == NULL )
		{
			head = foo;
			head->next = NULL;
		}
		else
		{
			// advace the pointer
			foo->next = head->next;
			head->next = foo;
		}
		cnt++;
	}
	else
	{
		K_LOG("invalid pointer in addNetEntry");
	}
}


/*
 * makePacket
 *
 * We first have to create a huge buffer that will hold our data.
 * Then we have to arrange the data in this buffer so that the
 * host_info_pkt is followed by the network_info packet for each
 * device. NOTE that we must be carefull to restore the info
 * and header information.. (well, just thier addresses at least :-)
 */
void host_info::makePacket()
{
	size_t len, total_len;
	struct nlist *foo = head;
	char *buff, *bar;

	/* first check to make sure we need to make the buff */
	if( (error != K_BAD) && (cnt != 0) && (head != NULL) )
	{
		/* calculate our needed size */
		total_len = sizeof(struct host_info_pkt);
		total_len += cnt * sizeof(struct network_info);
		buff = (char *)malloc(total_len);

		if( buff != NULL )
		{
			/* set our base */
			bar = buff;

			/* copy over the sys_info data */
			memcpy(buff, info, sizeof(struct host_info_pkt) );
			delete info;

			/* now re-set the length and the info */
			info = (struct host_info_pkt *)buff;
			header = (struct khneia_header_pkt *)info;
			info->header.length = total_len;

			bar += sizeof(struct host_info_pkt);
			/* now copy over all the other data */
			len = sizeof(struct network_info);

			/* make sure we do not exceed the length provided */
			while( (foo != NULL) && (bar < (buff + total_len)) )
			{
				memcpy(bar, &foo->info, len);
				foo = foo->next;
				bar += sizeof(struct network_info);
			}

		}// if buff

	} // if(error)... 

} // FIN


/*
 * Just delete the list and our buffer
 */
host_info::~host_info()
{
	struct nlist *foo = head;

	// delete the big buffer first
	free(info);

	if( head )
	{
		while( head->next != NULL )
		{
			foo = head;
			head = head->next;
			delete foo;
		}
		// now delete the last one
		delete head;
	}
}





