/**
 * Filename: client_listener.h
 *
 * Description: Header file that defines the client_listener module
 *
 * Original Author: Nathan Boeger
 *
 * $Id: client_listener.h,v 1.3 2002/04/26 08:31:07 nathan Exp $
 *
 */


class client_listener
{
public:

   /*
    * Default constructor will do nothing
    */
   client_listener();

   /*
    * This will create the listener and then recieve the 
    * proper config values from the client_sys_conf module;
    */
   client_listener(client_sys_conf *s);

   /*
    * run
    *
    * This will run the listener and start to listen. Once a packet is
    * recieved, it will call handle_pkt with the buffer recieved.
    */
   int run();

   /*
    * close 
    *
    * This will close the socket and clean up before
    * returing
    */
   int close();
   
   /*
    * CHILD SIGNAL HANDLER 
    *	
    * This will gather the childrens process info. Currently
    * the termination status or information is discarded
    */
   static void child_cleanup(int sig);

   /*
    * Destructor
    * no dynamic mem, does nothing
    */
   ~client_listener();

private:
   /*
    * handle_pkt
    *
    * This will fork and the child will service the packet, the parent will
    * return.
    */
   int handle_pkt(char *pkt);
   
   int error;
   int sfd;
   int nsfd;
   client_sys_conf *sys_conf;
};

