/**
* Filename: db_agent.h
*
* Description: database agent superclass
*
* Original Author: Nathan Boeger
*
* $Id: db_agent.h,v 1.6 2002/05/03 08:07:07 nathan Exp $
*
*/

#define MAX_CONSTRING (128)

class db_agent
{
public:

	/*
	 * default constructor does nothing
	 */
	db_agent();

	/*
	 * 
	 */
	db_agent(struct khneia_header_pkt *h, master_sys_conf *m);

	/*
	 * getClientId
	 * this will return the database client_id for this host. Otherwise
	 * it will return K_BAD on failure
	 */
	int getClientId();

	/*
	 * updateStatus
	 * Update status will update the poll_log and the poll tables with
	 * the reply just recieved. Note that in the poll log the timerecieved
	 * along with the notes (if applicable) will be updated. In the poll
	 * table the status weill be set to 0 for that pkt_type unless the
	 * REGISTER_SEQ is set then all packets for this client_id are set to 0
	 */
	int updateStatus();

	/*
	 * run 
	 * must be implemented in the drived class, This will 
	 * interact with the db and insert the data 
	 */
	virtual int run() = 0;

	/*
	 * report
	 * will analyze the data and report if its out of 
	 * allowed parameters. Must be implemented in the
	 * drived class 
	 */
	virtual int report() = 0;

	/*
	 * destructor
	 */
	virtual ~db_agent();

protected:

	/*
	 * createDBString()
	 * This will crate the dbstring for us to connect to the
	 * database
	 */
	int createDBString();

	int error; 
	int id; // database client id 
	char *dbuser;
	char *dbhost;
	char *dbname;
	char *dbpass;
	char dbconstring[MAX_CONSTRING];
	struct khneia_header_pkt *header;
	master_sys_conf *conf;
};

