/**
 * Filename: khneia_protocol.h
 *
 * Description: Header file that oulines the structures that are used
 * in the khneia protocol
 *
 * Original Author: Nathan Boeger
 *
 * $Id: khneia_protocol.h,v 1.15 2002/04/28 06:06:52 nathan Exp $
 *
 */

#define MAX_HOSTNAME_LEN     (64)
#define MAX_LOAD_LEN         (6)
#define MAX_OSINFO_LEN       (128)
#define MAX_VERSION_LEN      (32)
#define MAX_DEV_LEN          (16)
#define MAX_IPADDR           (16)
#define MAX_MAC_LEN          (32)
#define MAX_MOUNT_LEN        (128)
#define MAX_FS_LEN           (32)
#define MAX_MODULE_LEN       (128)
#define MAX_PATHLEN          (256)
#define MAX_CONNECTIONS      (128)


/* values for our packet types */
#define REGISTER_SEQ          (0x0)
#define HOST_INFO_PKT         (0x1)
#define SYSTEM_STATS_PKT      (0x2)
#define DISK_INFO_PKT         (0x4)
#define RSYNC_PKT             (0x5)  
#define SERVER_REQ_PKT        (0x6)
#define TASK_INITATE_PKT      (0x7)

/* other routienes */
#define LOAD_FORMAT           ("%2.2f")

typedef unsigned int  k_size_t;
typedef unsigned char k_type_t;
typedef float         k_float_t;

/*
 * khneia_header_pkt
 *
 * This is the main header used for all communication
 */
struct khneia_header_pkt 
{
   k_size_t sequence;
   k_type_t type;
   k_size_t length;
   char hostname[MAX_HOSTNAME_LEN];
};

/*
 * host_info_pkt
 * 
 * This packet is used to hold the information about the client. Since
 * a client can have multipe IP's, ethernet cards etc.. this packet
 * is not a fixed length. Instead there will be double pointer (**) 
 * to the information list
 */

struct network_info
{
   char ipaddr[MAX_IPADDR];
   char netmask[MAX_IPADDR];
   char mac[MAX_MAC_LEN];
   char dev[MAX_DEV_LEN];
};

struct host_info_pkt
{
   struct khneia_header_pkt header;
   char osinfo[MAX_OSINFO_LEN];
   char khneia_ver[MAX_VERSION_LEN];
   char def_route[MAX_IPADDR];
};


/*
 * hb_info_pkt
 *
 * The heart beat info packet will contain the data for a heart beat reply
 */
struct system_stats_pkt
{
   struct khneia_header_pkt header;
   char load_0[MAX_LOAD_LEN];
   char load_1[MAX_LOAD_LEN];
   char load_2[MAX_LOAD_LEN];
   k_size_t mem_total;
   k_size_t mem_used;
   k_size_t mem_free;
   k_size_t swap_total;
   k_size_t swap_used;
   k_size_t swap_free;
};

/*
 * disk_info_pkt
 *
 * This packet will remport the filesystem information. Since most systems
 * have multiple mount points. This will not have a fixed length, its just 
 * a shell for the disk_info buffers that are appneded to it
 */

struct disk_info
{
   char mount[MAX_MOUNT_LEN];
   char fstype[MAX_FS_LEN];
   k_size_t size;
   k_size_t used;
   k_size_t free;
};

struct disk_info_pkt
{
   struct khneia_header_pkt header;
};

/*
 * task_inipkt
 *
 * This packet is used to initiate the server check the database for pending
 * tasks 
 */
struct task_ini_pkt
{
   struct khneia_header_pkt header;
   k_size_t sequence;
};


/*
 * rsync_path_pkt
 *
 * This packet is sent from the server to the client with enough information
 * to call rsync
 */
struct rsync_path_pkt
{
   struct khneia_header_pkt header;
   char module[MAX_MODULE_LEN];
   char path[MAX_PATHLEN];
};


