/* 
 * Filename: master_poller.h
 *
 * Description: Header file for the master poller module
 *
 * Original Authoer: Nathan Boeger
 *
 * $Id: master_poller.h,v 1.2 2002/04/28 06:06:52 nathan Exp $
 *
 */
 #define MAX_HOSTNAME (32)
 
 class master_poller : public db_agent
 {
 public:

    /*
     * Default constructor does nothing
     */
    master_poller();

    /*
     * Prefered constructor
     * This will create the master poller with the sys_conf module
     * to gather the config option
     */
    master_poller(master_sys_conf *s);

    /*
     * run
     * This will run the poller component. The poller will at first 
     * set the poll interval (hbtime) then sleep for that amount of time.
     * When it wakes up, it will gather the hosts from the DB, fork
     * untill it has reached the MAXFORK limit or untill all hosts
     * have been contacted. If the MAXFORK value is reached. The 
     * poller will wait untill one if the children has exited, then
     * spawn more proccess and repeat untill all hosts have been
     * contacted. Run wil not return ! but if it does then its
     * value will be K_BAD :-) IE cannot fork
     */
    int run();
    
    /* does nothing */
    int report();

    /*
     * SIGNAL handler
     * Set the signal handler for the class,
     */
    static void fork_cleanup(int sig);

    /*
     * destructor, make sure oour list of hosts is properly deleted
     */
    ~master_poller();

 private:

    /*
     * Used to keep track of each host we need to connect to
     */
    struct khost 
    {
       char hostname[MAX_HOSTNAME];
       int sequence;
       int pkt_type;
       struct khost *next;
    } *head;

    /*
     * add_host
     * This will add the host to the classes linked list
     * of hosts.
     */
    int add_host(char *hname, int seq, int pkt);
    
    /*
     * clean_list
     * This will remove all hosts from our link list. Note that
     * this must be called before we add more :-)
     */
    int clean_list();
    /*
     * get_hosts
     * this will open a connection to the database and then
     * get all of the hosts in the 
     */
    int get_hosts();
    /*
     * send
     * This will be called by the child process only ! After the
     * send is complete it will then update the database for this
     * clients packet and exit.
     */
    int send(struct khost *host);

    /*
     * Set the maximum amount of forks we can do,
     * The default is  DEFAULT_MAXFORK 
     */
    int maxfork;
    /*
     * This will keep track of the children forked
     */
    int forkproc; 
    /*
     * This will control the interval inbtween each poll, this value
     * is only kept in secdonds for sleep.
     */
    int hbtime;
 };
