/**
 * Filename: master_sys_conf.h
 *
 * Description: header file for the master_sys_conf class definition
 *
 * Original Author: Nathan Boeger
 *
 * $Id: master_sys_conf.h,v 1.3 2002/04/28 06:06:52 nathan Exp $
 *
 */

/* max path len for the rsync path */
#define CONF_PATH     ("/usr/local/etc/khniea_master.conf")
#define DEF_DBNAME    ("khneia")
#define MAX_USERNAME  (128)
#define MAX_PASSWORD  (128)
#define MAX_DBNAME    (64)


class master_sys_conf
{

public:

	/*
	 * default constructor sets default values
	 */
	master_sys_conf();

	/*
	 * run the lexer and get the values
	 */
	int run();

	/*
	 * getError
	 * 
	 * This will return the state of the sys_conf module
	 * This allows us to check if we have run the parser
	 * or of the parser is in a bad state
	 */
	int getError();

	/*
	 * get / set Listenport
	 * will set or get the listen port for the server
	 */
	int getListenPort();
	int setListenPort(unsigned int p);

	/*
	 * get / set ClientPort
	 * will set or get the listen port for the server
	 */
	int getClientPort();
	int setClientPort(unsigned int p);

	/*
	 *	get / set DBServer port
	 * will set or get the database server port
	 */
	int getDBServerPort();
	int setDBServerPort(unsigned int p);

	/*
	 * get / set DBServer
	 * will get the server or set the server name for the
	 * database. Note that the server can be set by
	 * both an IP or valid hostname
	 */
	in_addr_t getDBServer();
	int setDBServer(in_addr_t s);
	int setDBServer(char *s);

	/*
	 * get / set DBUserName
	 * will get or set the database server username. 
	 */
	char *getDBUserName();
	int setDBUserName(char *n);

	/*
	 * get / set DBName
	 * will get or set the database name we need to connect to. 
	 */
	char *getDBName();
	int setDBName(char *n);

	/*
	 * get / set DBPassword
	 * will get or set the database server password.
	 * Now I know that passwords should not be stored
	 * plain text in config files... but what elese 
	 * can I do in the time alloted :-) 
	 */
	char *getDBPassword();
	int setDBPassword(char *p);

	/*
	 * get / set HBTime
	 * will get or set the heart beat time (values in secconds)
	 */
	int setHBTime(unsigned int t);
	int getHBTime();

	/*
	 * get /set maxfork
	 * This is the value for the poller. The default is 32 
	 */
	int setMAXFork(unsigned int mf);
	int getMAXFork();

	/*
	 * No dynmic mem.. nothing to get rid of
	 */
	~master_sys_conf();

private:

	// used to follow errors in the class
	int error;
	// database servers ip address
	in_addr_t dbserver;
	// database servers port
	unsigned int dbport;
	// database username
	char dbusername[MAX_USERNAME];
	// databases password
	char dbpassword[MAX_PASSWORD];
	// dataabse name
	char dbname[MAX_DBNAME];
	// the port we listen on
	unsigned int listenport;
	// the clients listen port
	unsigned int clientport;
	// heart beat time
	unsigned int hbtime;
	// max fork for the poller
	unsigned int maxfork;
};

