/**
 * Filename: khneia_sender.cpp
 *
 * Description: Class implementation for the sender module
 *
 * Original Author: Nathan Boeger
 *
 * $Id: khneia_sender.cpp,v 1.6 2002/05/03 08:07:07 nathan Exp $
 *
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <netinet/in.h>
#include <netinet/in.h>
#include <arpa/inet.h>


// my includes 
#include "flog.h"
#include "khneia.h"
#include "khneia_protocol.h"
#include "khneia_sender.h"

khneia_sender::khneia_sender()
{
	error = K_BAD;
}

khneia_sender::khneia_sender(struct khneia_header_pkt *h, int p, in_addr_t s)
{
	if( h )
	{
		// Set our info
		header = h;
		// we set this to a proper network bite ordered port
		port = htons(p);
		server = s;
		error = K_GOOD;
		error = setHostName();
	}
	else
	{
		// will be bad by default but.. 
		error = K_BAD;
	}
}

int khneia_sender::send()
{
	int sfd;
	struct sockaddr_in him;
	k_size_t len;

	// test the error !!
	if( error != K_BAD )
	{
		// now create the socket
		if( (sfd = socket(AF_INET, SOCK_STREAM, 0) ) > 0 )
		{
			// prepare our structure
			memset(&him, 0, sizeof(struct sockaddr_in) );
			him.sin_family = AF_INET;
			him.sin_port = port;
			him.sin_addr.s_addr = server;

			// conect
			if( connect(sfd, (struct sockaddr *)&him, sizeof(struct sockaddr_in) ) >= 0 )
			{
				/* get our len before conversion */
				len = header->length;
				/* convert our needed data to network byte order */
				H_TO_NWL(header->length);
				H_TO_NWL(header->sequence);
				write(sfd, header, len);
				shutdown(sfd, SHUT_RDWR);
				close(sfd);
			} // if(connect)
			else
			{
				K_ERR_LOG("SENDER");
			}  

		} // if( socket)
		else
		{
			K_ERR_LOG("SENDER");
		}
	}// if( error )
	else
	{
		K_LOG("SENDER");
	}

	return(error);
}

/*
 * setHostName
 *
 * This will just call gethostname and then copy
 * over the allowd amount to the hostname entry
 * in the packet.
 */
int khneia_sender::setHostName()
{
	int err = error;
	// make a buffer big enough to hold a long hostname 256 char
	char temp[MAX_HOSTNAME_LEN * 8];
	size_t len = MAX_HOSTNAME_LEN;

	// check our erro conditions
	if( error != K_BAD )
	{
		// zero out our structs
		memset(temp, 0, (len * 8) );
		memset(header->hostname, 0, len);

		if( gethostname(temp, len) < 0 )
		{
			K_ERR_LOG("gethostname");
			strncpy(header->hostname, "NOT FOUND", strlen("NOT FOUND"));
			err = K_BAD;
		}
		else
		{
			strncpy(header->hostname, temp, len);
			// null it !
			if( strlen(temp) > len )
				header->hostname[MAX_HOSTNAME_LEN - 1] = '\0';
		}
	}

	return(error);
}


/* no dynamic mem, do nothing */
khneia_sender::~khneia_sender()
{

}
