/**
 * Filename: driver.cpp
 *
 * Description: Driver file for testing the classes
 *
 * Original Author: Nathan Boeger
 *
 * $Id: driver.cpp,v 1.9 2002/05/07 06:50:52 nathan Exp $
 *
 */

#include <stdio.h> 
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <signal.h>
#include <setjmp.h>


#include "khneia.h"
#include "khneia_protocol.h"
#include "master_sys_conf.h"
#include "db_agent.h"
#include "master_poller.h"
#include "master_listener.h"
#include "flog.h"

/* our signal handler */
static void reset(int sig);
void master();

/*
 * pids for our children.. since we need to keep 
 * track of them for when we get the HUP signal
 */
pid_t listen_pid, poller_pid;

/* globals */
master_sys_conf *sys  = NULL;
master_listener *lis  = NULL;
master_poller   *poll = NULL;

/* 
 * save our program enviroment for when we 
 * recieve a signal (in our case here HUP)
 */
jmp_buf jbuf;

int main()
{
	if( fork() == 0 )
	{
		/* make me a deamon */
		setsid();
		/* work from root to make things simple :-) */
		chdir("/");
		/* we catch SIGHIUP and reset the system */
		signal(SIGHUP, reset);

		/*
		 * Check to see if we have already 
		 * had a signal !
		 */
		if( setjmp(jbuf) == 0 )
		{
			master();
		}
		else
		{
		  /*
			* To reset the system (conf file changes etc... )
			* just delete our current class instances and call
			* listens close to cleanup the socket. This will
			* be executed only after a return from a signal
			*/

			delete sys;
			sys = NULL;

			master();
		}
	}

	return(0);
}

/*
 * master
 *
 * Spawn off the children to listen and
 * poll the clients.
 */
void master()
{
	sys = new master_sys_conf();

	if( (sys) && (sys->run() >= 0) )
	{
		listen_pid = fork();
		if( listen_pid == 0 )
		{
			// child 
			signal(SIGHUP, SIG_IGN);
			lis = new master_listener(sys);
			lis->run();
			// never reached.. 
			exit(-1);
		}

		poller_pid = fork();
		if( poller_pid == 0 )
		{
			// child 
			signal(SIGHUP, SIG_IGN);
			poll = new master_poller(sys);
			poll->run();
			// never reached.. 
			exit(-1);
		}
	}
	/*
	 * we wait like a good processes.. but they should
	 * not return !
	 */
	waitpid(-1, NULL, 0);
}


/*
 * reset
 * Once we get the HUP signale, we need to kill our children
 * processes (tech talk is so morbid !) then we have to restart
 * them all over. This is not very graceful, and later we should
 * send them a SIGUSER so they can clean up themselfs.
 */
static void reset(int sig)
{

	// ouch...
	kill(poller_pid, SIGKILL);
	waitpid(-1, NULL, 0);
	
	kill(listen_pid, SIGKILL);
	waitpid(-1, NULL, 0);

	// go back and do agian
	longjmp(jbuf, 1);
}

