/**
 * Filename: master_sys_conf.cpp
 *
 * Description: master_sys_conf implementation, note that this
 * file is *not* directly compiled.. it is manifested
 * into the master_sys_conf.l file !!!
 *
 * Original Author: Nathan Boeger
 *
 * $Id: master_sys_conf.cpp,v 1.5 2002/04/28 06:06:52 nathan Exp $
 *
 */

/*
 * Default constructor
 *
 * This will just set the serverport and
 * the listen port to the defaults, and flag
 * the error to good
 */
master_sys_conf::master_sys_conf()
{
	strncat(dbname, DEF_DBNAME, MAX_DBNAME);
	error = K_BAD;
	listenport = SERVERPORT;
	clientport = CLIENTPORT;
	maxfork = DEFAULT_MAXFORK;
}

/*
 * run
 * ---
 *
 * This will the lexer and gather the options. If it is
 * sucessfull it returns K_GOOD, otherwise K_BAD
 */
int master_sys_conf::run()
{
	{
		FlexLexer* lexer;
		ifstream *in;
		ofstream *out;

		in = new ifstream(CONF_PATH);
		out = new ofstream(OUT_FILE, ios::nocreate);

		// this will allow us to have multiple instances of this class
		sys_lexer = this; 

		if( !in->bad() && !out->bad() )
		{
			lexer = new master_sys_confFlexLexer(in, out);
			if( lexer )
			{
				while( lexer->yylex() != 0 ) ;
				delete lexer;
				error = K_GOOD;
			}
			delete in;
			delete out;
		}

		/* 
		 * return the class error
		 * 
		 * when the lexer runs it will populate data by calling
		 * the set methods. If they have errors they will flag
		 * the class error
		 */
		return(error);
	}
}

/*
 * getError
 *
 * Just return the status of the error (later will create an
 * error code ERR_HASNOT_RUN) but for now... 
 */
int master_sys_conf::getError()
{
	return(error);
}

/*
 * getlistenPort
 *
 * This will return the port that we need to listen
 * on, The default is 8889
 */
int master_sys_conf::getListenPort()
{
	return(listenport);
}

/*
 * setListenPort
 *
 * This will set the clients servers port. Default is 8889
 */
int master_sys_conf::setListenPort(unsigned int p)
{
	listenport = p;
	return(listenport);
}

/*
 * getClientPort
 *
 * This will return the port that the clients are listening on
 * on, The default is 8888
 */
int master_sys_conf::getClientPort()
{
	return(clientport);
}

/*
 * setListenPort
 *
 * This will set the clients listen port. Default is 8888
 */
int master_sys_conf::setClientPort(unsigned int p)
{
	clientport = p;
	return(clientport);
}

/*
 * getDBServerPort
 *
 * This will return the database servers port we need
 * to connect to
 */
int master_sys_conf::getDBServerPort()
{
	return(dbport);
}

/*
 * setServerPort()
 *
 * This will set the database server port that we need to connect to
 */
int master_sys_conf::setDBServerPort(unsigned int p)
{
	dbport = p;
	return(dbport);
}

/*
 * getServer
 *
 * This will return the IP address of the database server that we need
 * to connect to.
 */
in_addr_t master_sys_conf::getDBServer()
{
	return(dbserver);
}

/*
 * setServer(in_addr_t s)
 * sets the database server to the ip address s
 */
int master_sys_conf::setDBServer(in_addr_t s)
{
	dbserver = s;
	return(error);
}


/*
 * setServer(in_addr_t s)
 * sets the database server to the ip address s
 */
int master_sys_conf::setDBName(char *n)
{
	if( n )
	{
		strncat(dbname, n, MAX_DBNAME);
	}
	return(error);
}

/*
 * setServer(in_addr_t s)
 * sets the database server to the ip address s
 */
char *master_sys_conf::getDBName()
{
	return(dbname);
}


/*
 * setDBServer(char *s)
 *
 * This will take the servername s and convert it to
 * an IP address. Note that if the name does not 
 * resolve then we throw the error for the whole
 * Otherwise its just good
 * 
 */
int master_sys_conf::setDBServer(char *s)
{
	struct hostent *hinfo = NULL;

	if( s )
	{
		hinfo = gethostbyname(s);
		if( hinfo )
		{
			// now set the server from the info
			memcpy(&dbserver, hinfo->h_addr_list[0], hinfo->h_length );
			error = K_GOOD;
		}
		else
		{
			error = K_BAD;
		}
	}

	return(error);
}

/*
 * getDBUserName
 *
 * This will return the database username
 */
char *master_sys_conf::getDBUserName()
{
	return(dbusername);
}

/*
 * setDBUserName
 *
 * This will set the path to the rsync binary
 */
int master_sys_conf::setDBUserName(char *n)
{
	/* check the pointers !! */
	if( n )
	{
		memset(dbusername, 0, MAX_PATH_LEN);
		strncat(dbusername, n, strlen(n) );
	}
	else
	{
		error = K_BAD;
	}

	return(error);
}

/*
 * getDBUserName
 *
 * This will return the database username
 */
char *master_sys_conf::getDBPassword()
{
	return(dbpassword);
}

/*
 * setDBUserName
 *
 * This will set the path to the rsync binary
 */
int master_sys_conf::setDBPassword(char *p)
{
	/* check the pointers !! */
	if( p )
	{
		memset(dbpassword, 0, MAX_PATH_LEN);
		strncat(dbpassword, p, strlen(p) );
	}
	else
	{
		error = K_BAD;
	}

	return(error);
}


/*
 * getHBTime
 *
 * This will return the time in secconds for the heartbeats
 */
int master_sys_conf::getHBTime()
{
	return(hbtime);
}

/*
 * setHBTime
 *
 * This will set the time in secconds for the heartbeat
 */
int master_sys_conf::setHBTime(unsigned int t)
{
	hbtime = t;
	return(t);
}

/*
 * setMAXFork
 * This will set the value for maxfork used by the poller.
 */
int master_sys_conf::setMAXFork(unsigned int mf)
{
	maxfork = mf;
	return(maxfork);
}

/*
 * getMAXFork
 * Return the maxfork vaule
 */
int master_sys_conf::getMAXFork()
{
	return(maxfork);
}

/*
 * Destructor
 */
master_sys_conf::~master_sys_conf()
{
	// no memory allocated
}

