%{
/**
 * Filename: sys_conf.l
 *
 * Description: the lexer used to parse the khneia
 * client conf file. This will manifest the files
 * that implement the class !
 *
 * Original Author: Nathan Boeger
 *
 * $Id: master_sys_conf.l,v 1.3 2002/04/28 06:06:52 nathan Exp $
 *      
 */

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <fstream.h>
#include <string.h>
#include <sys/types.h>
#include <syslog.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>


 // my includes
#include "khneia.h"
// the  client_conf header
#include "master_sys_conf.h"
master_sys_conf *sys_lexer;

/*
 * The lexer directives are pretty simple. I have accounted for options that are
 * not case sensitive (eg: MAXFORK, MaxFoRK, maxFORK etc..) will all be parsed
 * correctly. This does have some overhead but since the config file is pretty
 * small its not a real issue. Also for clarity I have made specific lexer directives
 * even though they could be covered as one (ex: int could be used for port, 
 * maxfork, and other interger based config values). This was done to allow a 
 * greater amount of flexibility. Changing the port directive would only require
 * changing the port directive.
 */
%}

%option prefix="master_sys_conf"
%option noyywrap

dbserver     [ \t]*[dD][bB][sS][eE][rR][vV][eE][rR][ \t]*"="
dbport       [ \t]*[dD][bB][pP][oO][rR][tT][ \t]*"="
dbuser       [ \t]*[dD][bB][uU][sS][eE][rR][ \t]*"="
dbname       [ \t]*[dD][bB][nN][aA][mM][eE][ \t]*"="
dbpass       [ \t]*[dD][bB][pP][aA][sS][sS][ \t]*"="
l_port       [ \t]*[lL][iI][sS][tT][eE][nN][pP][oO][rR][tT][ \t]*"="
hbtime       [ \t]*[hH][bB][tT][iI][mM][eE][ \t]*"="
c_port       [ \t]*[cC][lL][iI][eE][nN][tT][pP][oO][rR][tT][ \t]*"="
mfork        [ \t]*[mM][aA][XxT][Ff][oO][rR][kK][ \t]*"="

ip         ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)
val        [0-9a-zA-Z\./\-_]*
hostname   ([0-9a-zA-Z\._\-]*)
port       [0-9]+
ktime      [0-9]+
forkval    [0-9]+
comment    \n?|([ \t]*)?#.*\n
endval     .*\n    

%x GET_DBSERVER
%x GET_DBPORT
%x GET_DBUSER
%x GET_DBNAME
%x GET_DBPASS
%x GET_L_PORT
%x GET_C_PORT
%x GET_HBTIME
%x GET_MFORK
%x GET_IP
%x GET_VAL
%x END_VAL 

%%


^{comment}               { ; /* ignore comments */ }

^{dbserver}              { BEGIN GET_DBSERVER;   }
<GET_DBSERVER>{ip}       { sys_lexer->setDBServer( inet_addr(yytext) ); BEGIN END_VAL; }
<GET_DBSERVER>{hostname} { sys_lexer->setDBServer( yytext);             BEGIN END_VAL; }

^{dbport}                { BEGIN GET_DBPORT; }
<GET_DBPORT>{port}       { sys_lexer->setDBServerPort( atoi(yytext) );  BEGIN END_VAL; }

^{dbuser}                { BEGIN GET_DBUSER; }
<GET_DBUSER>{val}        { sys_lexer->setDBUserName( yytext );          BEGIN END_VAL; }

^{dbpass}                { BEGIN GET_DBPASS; }
<GET_DBPASS>{val}        { sys_lexer->setDBPassword( yytext );          BEGIN END_VAL; }

^{dbname}                { BEGIN GET_DBNAME; }
<GET_DBNAME>{val}        { sys_lexer->setDBName( yytext );              BEGIN END_VAL; }

^{hbtime}                { BEGIN GET_HBTIME; }
<GET_HBTIME>{ktime}      { sys_lexer->setHBTime( atoi(yytext) );        BEGIN END_VAL; }

^{l_port}                { BEGIN GET_L_PORT; }
<GET_L_PORT>{port}       { sys_lexer->setListenPort( atoi(yytext) );    BEGIN END_VAL; }

^{c_port}                { BEGIN GET_C_PORT; }
<GET_C_PORT>{port}       { sys_lexer->setClientPort( atoi(yytext) );    BEGIN END_VAL; }

^{mfork}                 { BEGIN GET_MFORK;  }
<GET_MFORK>{forkval}     { sys_lexer->setMAXFork( atoi(yytext) );       BEGIN END_VAL; }

<END_VAL>{endval}        { BEGIN 0; }
.|\n                     { ; /* Ignore white space */ }

%%

// the client_conf implementation
#include "master_sys_conf.cpp"
